#include<linux/init.h>
#include<linux/module.h>
#include<linux/fs.h>
#include<linux/string.h>
#include<linux/uaccess.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("John Doe");
MODULE_DESCRIPTION("Semplice Linux character device driver");

static int device_file_major_num = 0;
static const char device_name[] = "driver_facile";
static int ret = 0;

static char driver_buffer[100];   			//buffer locale del driver

static int driver_facile_open(struct inode *inode, struct file *file);

static int driver_facile_release(struct inode *inode, struct file *file);

static ssize_t driver_facile_write(struct file *file, const char *buff, size_t len, loff_t *off);

static ssize_t driver_facile_read(struct file *file, char *buff, size_t len, loff_t *off);

//struttura contenete le operazioni su file
static struct file_operations fops=
{
	.open = driver_facile_open,   		//puntatore alla funzione open del device
	.release = driver_facile_release, 		//puntatore alla funzione open del device
	.write = driver_facile_write, 		//puntatore alla funzione open del device
	.read = driver_facile_read, 			//puntatore alla funzione open del device
};

static int driver_facile_init(void)   		//funzione di init chiamata da insmod
{
	printk("driver_facile: Buongiorno kernel\n"); 
	ret = register_chrdev(device_file_major_num, device_name, &fops);
	if(ret < 0)
	{
		printk(KERN_ALERT "driver_facile: Registrazione del device fallita!\n");
	}
	else
	{
		device_file_major_num = ret;
		printk(KERN_ALERT "driver_facile: Registrazione del device con major num: %i\n", device_file_major_num);
	}
	return 0;
}

static void driver_facile_exit(void) 			//funzione di exit chiamata da rmmmod
{
	if(device_file_major_num > 0)
	{
		unregister_chrdev(device_file_major_num, device_name);
		printk("driver_facile: Arrivederci Kernel\n");
	}
}
 
static int driver_facile_open(struct inode *inode, struct file *file)
{
	printk("driver_facile: Apertura del device!\n");
	return 0;
}

static int driver_facile_release(struct inode *inode, struct file *file)
{
	printk("driver_facile: Chiusura del device!\n");
	return 0;
}

static ssize_t driver_facile_write(struct file *file, const char *buff, size_t len, loff_t *off)
{
	ret = copy_from_user(driver_buffer, buff, len);
	if(ret != 0)
	{
		printk(KERN_ALERT "driver_facile: La funzione di write non ha funzionato corretamente!\n");
	}
	driver_buffer[len]=0;
	return ret;
}

static ssize_t driver_facile_read(struct file *file, char *buff, size_t len, loff_t *off)
{
	ret = copy_to_user(buff, driver_buffer, len);
	if(ret != 0)
	{
		printk(KERN_ALERT "driver_facile: La funzione di read non ha funzionato corretamente!\n");
	}
	return ret;
}

module_init(driver_facile_init);
module_exit(driver_facile_exit);
